#! /usr/bin/perl
#
# cod_stat.pl - statistics generator for Call of Duty (and MoHAA?)
# Copyright (C) 2005 Tommi Saviranta <wnd@iki.fi>
#
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses>.
#
# Version: cod_stat.pl v0.3.2  19-Mar-2005  wnd@iki.fi

use strict;
use warnings FATAL => 'all';

my $base_rating = 100;

do "db.pl";
do "common.pl";



my ($map, $gametype, $starttime);	# Current map/gametype, start time
my ($fb_killer, $fb_victim, $fb_time, $fb_weapon);	# First blood
my (@actionlog);			# Stuff that's going on

my @game_to_uniq;			# [game_id] = uniq_id
my @uniq_nicks;				# [uniq_id] = \n nick \n [nick \n] ...
my @join_time;				# [uniq_id] = join time
my @play_time;				# [uniq_id] = play time

my @spree_current;			# [uniq_id] = current killing spree
my @spree_best;				# [uniq_id] = best killing spree
my @spree_reason;			# [uniq_id] = spree end reason
					# "uniq_id \n uniq_id" = killed
					# "" = game ends

my $uniq_id;
my $game_state = 0;			# Set at first "end game" event
					# 0 not started
					# 1 running
					# 2 finished

# deaths include suicides
my %kills;				# uniq_id \n uniq_id => kills
					# uniq_id => kills
my $have_kills = 0;			# suicides don't count
					# used to define if a game was played



open(FD, "< $ARGV[0]") or die "Cannot open $ARGV[0]";

# Main loop: read input
while (1) {
	my $tmp;
	my ($time, $min, $sec);
	my (@stuff, $action, $type);

	$_ = <FD>;
	if (! defined($_)) {
		print ".";
		sleep(30);
		next;
	}
	
	# Strip bad stuff, ignore empty lines
	s/\^+\d+//g; tr/$%@'/s\-+\-/;
	next if (/^$/);
	
	# Split line
	($min, $sec, $action) = /\s*(\d+):(\d+)\s+(.*)/;
	$time = $min * 60 + $sec;
	@stuff = split(/;/, $action);
print "$time: $action\n";
	$type = $stuff[0];

	if ($type eq "D") {			# damage
		parse_damage($time, @stuff);
	}
	
	elsif ($type eq "K") {			# kill
		parse_kill($time, @stuff);
	}
	
	elsif ($type eq "say") {		# say
		parse_say($time, @stuff);
	}
	
	elsif ($type eq "sayteam") {		# sayteam
		parse_sayteam($time, @stuff);
	}
	
	elsif ($type eq "J") {		# join
		parse_join($time, @stuff);
	}
	
	elsif ($type eq "Q") {		# quit
		parse_quit($time, @stuff);
	}

	elsif ($type eq "W") {			# win
		parse_win($time, @stuff);
	}
	
	elsif ($type eq "L") {			# lose
		parse_lose($time, @stuff);
	}
	
	elsif ($type eq "A") {			# action
		parse_action($time, @stuff);
	}
	
	elsif ($type eq "Weapon") {		# collect weapon? XXX
	}
	
	elsif ($type eq "Item") {		# collect item
	}
	
	elsif ($type =~ /^----/) {		# separator
	}
	
	elsif ($type =~ /^ExitLevel/) {		# end map
		parse_end_game($time);
	}
	
	elsif ($type =~ /^InitGame/) {		# new map
		($tmp) = $action =~ /.*?: (.*)/;
		parse_init_game($time, $tmp);
	}
	
	elsif ($type =~ /^Shutdown/) {		# server shutdown
		parse_end_game($time);
	}
	
	elsif ($type =~ /^RestartGame/) {	# game restart
		parse_end_game($time);
	}
	
	else {
		print "Unknown action/line: '$action'\n";
	}
}



# parse_init_game:
# 0	time
# 1	stuff
#
sub parse_init_game {
	my $time = $_[0];
	my @stuff;

	if ($game_state == 1) {
		print "Finishing old game first...\n";
		parse_end_game($time);
	}

	undef @game_to_uniq;
	undef @uniq_nicks;
	undef @join_time;
	undef @play_time;

	undef $uniq_id;

	undef %kills;

	undef @spree_current;
	undef @spree_best;
	undef @spree_reason;
	
	undef $fb_killer;
	undef $fb_victim;
	undef $fb_time;
	undef $fb_weapon;

	$starttime = $time;
	$uniq_id = 0;

	@stuff = split(/\\/, $_[1]);
	$gametype = get_field("g_gametype", @stuff);
	$map = get_field("mapname", @stuff);

	$game_state = 1;
	$have_kills = 0;
}



# get_field:
# 0	field
# 1...	stuff
#
sub get_field {
	my $i;
	for ($i = 1; $i <= $#_ - 1; $i++) {
		if ($_[$i] eq $_[0]) {
			return $_[$i + 1];
		}
	}
	return "";
}



# parse_end_game:
# 0	time
#
sub parse_end_game {
	my $time = $_[0];
	my @uniq_to_real;
	my %real_players_tmp;
	my @real_players;
	my ($i, $j);
	my $real_id;

	return if (! defined($uniq_id) || $game_state == 2);
print "Too short game\n" if ($time - $starttime < 60);
print "Unpopulated game\n" if ($uniq_id < 2);
print "Game with no kills\n" if ($have_kills == 0);
	return if ($have_kills == 0
			|| $uniq_id < 2
			|| ($time - $starttime < 60));
	
	# Get real player ids
	for ($i = 0; $i < $uniq_id; $i++) {
		my (%uniq_hash, @nicks);
		my $last = "";	# speed up -- avoid matching and hashing
		
		foreach $_ (split(/\n/, $uniq_nicks[$i])) {
			next if ($_ eq $last);
			next if (/^$/);
			next if ($_ eq "Unknown Soldier");
			$uniq_hash{$_} = 1;
			$last = $_;
		}
		foreach $_ (keys %uniq_hash) {
			push(@nicks, $_);
		}
		$real_id = get_player_id(@nicks);
		$uniq_to_real[$i] = $real_id;
		$real_players_tmp{$real_id} = 1;
	}
	foreach $_ (keys %real_players_tmp) {
		push(@real_players, $_);
	}

	# Build stats for real players
	my %real_play_time;
	my (%real_kills, %real_deaths, %real_suicides);
	my ($killer, $victim, $t);
	for ($i = 0; $i < $uniq_id; $i++) {
		$killer = $uniq_to_real[$i];

		update_spree($i, 0, "");
		player_leave($time, $i);

		# Count playtime
		$real_play_time{$killer} += $play_time[$i];

		# Get kills
		for ($j = 0; $j < $uniq_id; $j++) {
			$victim = $uniq_to_real[$j];
			$t = $kills{"$i\n$j"};
			$t = 0 if (! defined($t));
			$real_kills{"$killer\n$victim"} += $t;
			if ($killer == $victim) {
				$real_suicides{"$killer"} = $t;
			} else {
				$real_kills{"$killer"} += $t;
			}
			$real_deaths{"$victim"} += $t;
		}
	}

	my ($game_id, $gametype_id) = create_game($gametype, $map,
			$time - $starttime, $#real_players + 1);

	# Stats
	my ($s_kills, $s_deaths, $s_suicides);
	my ($s_time, $s_kph, $s_ttl);
	my ($s_rating, $s_rating_ch);
	# Stats cache
	my (%c_kills, %c_deaths, %c_suicides);
	my (%c_time, %c_rating_ch);
	
	my ($killer_rating, $victim_rating);
	my ($killer_score, $victim_score);
printf("\n\nGame $game_id\n\n\n");
	foreach $killer (@real_players) {
		$killer_rating = get_player_rating($killer, $gametype_id);
		$s_rating_ch = 0;
		# Rating
		foreach $victim (@real_players) {
			my $kills;
			$victim_rating = get_player_rating($victim,
					$gametype_id);
			$kills = Common::safe_number(
					$real_kills{"$killer\n$victim"});
			$killer_score = $kills * $victim_rating;
			$victim_score = Common::safe_number(
					$real_kills{"$victim\n$killer"})
				* $killer_rating;
			if ($killer != $victim) {
				$s_rating_ch += ($killer_score - $victim_score)
						/ $killer_rating;
			} else {
				# 5 %. minus for each suicide
				$s_rating_ch -= ($killer_rating / 200 * $kills);
			}

			my $query = "INSERT INTO game_kills" .
				" (game, killer, victim, kills)" .
				" VALUES" .
				" ($game_id, $killer, $victim, $kills)";
			DB::update($query);
		}
		
		$s_kills = Common::safe_number($real_kills{$killer});
		$s_deaths = Common::safe_number($real_deaths{$killer});
		$s_suicides = Common::safe_number($real_suicides{$killer});
		
		$s_time = $real_play_time{$killer};
		if ($s_time != 0) {
			$s_kph = $s_kills * 3600 / $s_time;
		} else {
			$s_kph = 0;
		}
		$s_ttl = $s_time / ($s_deaths + 1);
		$s_rating = $killer_rating + $s_rating_ch;

		$c_kills{$killer} = $s_kills;
		$c_deaths{$killer} = $s_deaths;
		$c_suicides{$killer} = $s_suicides;
		$c_time{$killer} = $s_time;
		$c_rating_ch{$killer} = $s_rating_ch;

		next if ($killer == -1);

		my $query;
		$query = "INSERT INTO game_player" .
				" (game, player, kills, deaths," .
				" suicides, rating, rating_change, time)" .
				" VALUES" .
				" ($game_id, $killer, $s_kills, $s_deaths," .
			" $s_suicides, $s_rating, $s_rating_ch, $s_time)";
		DB::update($query);
	}

	# Update stats
	my $total_kills = 0;
	my $total_suicides = 0;
	my ($rating_low, $rating_high);
	foreach $killer (@real_players) {
		next if ($killer == -1);
		my ($p_kills, $p_deaths, $p_suicides);
		my ($p_time, $p_games, $p_rating);
		my ($query, $result, @row);
		
		$query = "SELECT kills, deaths, suicides," .
				" time, games, rating" .
				" FROM player_stat WHERE player = $killer" .
					" AND gametype = $gametype_id";
		$result = DB::query($query);
		@row = $result->fetchrow;
		
		$p_kills	= $row[0] + $c_kills{$killer};
		$p_deaths	= $row[1] + $c_deaths{$killer};
		$p_suicides	= $row[2] + $c_suicides{$killer};

		$p_time		= $row[3] + $c_time{$killer};
		$p_games	= $row[4] + 1;
		$p_rating	= $row[5] + $c_rating_ch{$killer};

		$query = "UPDATE player_stat SET".
				" kills = $p_kills," .
				" deaths = $p_deaths," .
				" suicides = $p_suicides," .
				" time = $p_time," .
				" games = $p_games," .
				" rating = $p_rating" .
				" WHERE player = $killer" .
					" AND gametype = $gametype_id";
		DB::update($query);

		$total_kills += $c_kills{$killer};
		$total_suicides += $c_suicides{$killer};
		$rating_low = $p_rating if (! defined($rating_low)
				|| $p_rating < $rating_low);
		$rating_high = $p_rating if (! defined($rating_high)
				|| $p_rating > $rating_high);
	}

	# Update gametype stats
	my ($query, $result, @row);
	$query = "SELECT kills, suicides, games, time," .
		" rating_low, rating_high FROM gametype" .
		" WHERE id = $gametype_id";
	$result = DB::query($query);
	@row = $result->fetchrow;
	$total_kills += $row[0];
	$total_suicides += $row[1];
	my $total_games = $row[2] + 1;
	my $total_time = $row[3] + ($time - $starttime);
	$rating_low = $row[4] if ($row[4] < $rating_low);
	$rating_high = $row[5] if ($row[5] > $rating_high);

	$query = "UPDATE gametype SET " .
			" kills = $total_kills, suicides = $total_suicides," .
			" games = $total_games, time = $total_time," .
			" rating_low = $rating_low," .
			" rating_high = $rating_high" .
			" WHERE id = $gametype_id";
	DB::update($query);

	# Update quick_info
	my $map_id = get_map_id($map);
	$query = "UPDATE quick_info SET" .
			" last_game = $game_id," .
			" last_map = '$map'," .
			" last_map_id = $map_id," .
			" last_gt = '$gametype'," .
			" last_gt_id = $gametype_id," .
			" last_players = " . ($#real_players + 1);
	DB::update($query);

	$game_state = 2;
}



# safe_set:
# 0	value
#
sub safe_set {
	if (defined($_[0])) {
		return $_[0];
	} else {
		return 0;
	}
}



# get_player_id:
# 0...	player nick
# ret	player id
#
sub get_player_id {
	my ($query, $result, @row);
	my $i;
	my $id;

	return -1 if ($#_ == -1);	# anonymous coward -> id == -1
	
	$query = "SELECT id FROM player_alias WHERE name = '$_[0]'";
	for ($i = 1; $i <= $#_; $i++) {
		$query .= " OR name = '$_[$i]'";
	}

	$result = DB::query($query);
	@row = $result->fetchrow;
	if ($#row == -1) {
		my $seq;
		# Not in database: create new player
		DB::transaction_start();
		$id = DB::get_sequence("player_id_seq");
		$query = "INSERT INTO player (id, name) VALUES ($id, '$_[0]')";
		DB::update($query);
		DB::transaction_end();
	} else {
		$id = $row[0];
	}

	update_aliases($id, @_);
	return $id;
}



# update_aliases:
# 0	id
# 1...	nick
#
sub update_aliases {
	my $id = $_[0];
	my ($query, $result, @row);
	my $i;
	
	for ($i = 1; $i <= $#_; $i++) {
		$query = "SELECT id FROM player_alias WHERE name = '$_[$i]'";
		$result = DB::query($query);
		@row = $result->fetchrow;
		if ($#row == -1) {
			$query = "INSERT INTO player_alias (name, id) VALUES"
					. " ('$_[$i]', $id)";
			DB::update($query);
		}
	}
}



# parse_join:
# 0	time
# 1...	stuff
#
sub parse_join {
	my ($time, $player, $id, $team, $save_id);
	
	$time = $_[0];
	$team = $_[1];
	if ($#_ == 4) {
		$player = $_[4];
		$id = $_[3];
	} else {
		$player = $_[3];
		$id = $_[2];
	}

	$game_to_uniq[$id] = $uniq_id;
	$uniq_nicks[$uniq_id] = "\n$player\n";
	$join_time[$uniq_id] = $time;
	$play_time[$uniq_id] = 0;

	$spree_current[$uniq_id] = 0;
	$spree_best[$uniq_id] = 0;
	
	$uniq_id++;
}



sub parse_quit {
	my ($time, $player, $id);
	my $uid;
	
	$time = $_[0];
	if ($#_ == 4) {
		$player = $_[4]; $id = $_[3];
	} else {
		$player = $_[3]; $id = $_[2];
	}

	$uid = $game_to_uniq[$id];
	# What is this? There are players in who never joined the game?
	return if (! defined($uid));
	$uniq_nicks[$uid] .= "$player\n";
	player_leave($time, $uid);
}



# player_leave:
# 0	time
# 1	uniq_id
#
sub player_leave {
	my ($time, $uid) = @_;

	return unless (defined($join_time[$uid]));
	$play_time[$uid] = $time - $join_time[$uid];
	undef $join_time[$uid];
}



sub parse_win {
}



sub parse_lose {
}



sub parse_say {
}



sub parse_sayteam {
}



sub parse_damage { # TODO
}



# parse_kill:
# 0	time
# 1...
#
sub parse_kill {
	my $time = $_[0];
	
	my ($killer_id, $killer_uniq, $killer_nick, $killer_team);
	my ($victim_id, $victim_uniq, $victim_nick, $victim_team);
	
	my ($weapon, $dmg_type, $dmg_loc);
	
	if ($#_ == 13) {
		$killer_nick = $_[9]; $killer_id = $_[7]; $killer_team = $_[8];
		$victim_nick = $_[5]; $victim_id = $_[3]; $victim_team = $_[4];
		$weapon = $_[10]; $dmg_type = $_[12]; $dmg_loc = $_[13];
	} else {
		$killer_nick = $_[7]; $killer_id = $_[5]; $killer_team = $_[6];
		$victim_nick = $_[4]; $victim_id = $_[2]; $victim_team = $_[3];
		$weapon = $_[8]; $dmg_type = $_[10]; $dmg_loc = $_[11];
	}
	
	$killer_uniq = $game_to_uniq[$killer_id];
	$victim_uniq = $game_to_uniq[$victim_id];

	$uniq_nicks[$killer_uniq] .= "$killer_nick\n";
	$uniq_nicks[$victim_uniq] .= "$victim_nick\n";

	$kills{"$killer_uniq\n$victim_uniq"}++;
	if ($killer_id != $victim_id && ! defined($fb_killer)) {
		$fb_killer = $killer_uniq;
		$fb_victim = $victim_uniq;
		$fb_time = $time;
		$fb_weapon = $weapon;

		update_spree($killer_uniq, 1);
		$have_kills = 1;
	}
	update_spree($victim_uniq, 0, "$killer_id\n$weapon");
}



# update_spree:
# 0	player
# 1	action: 0 == end spree, 1+ == got score
# (2)	spree end reason
#
sub update_spree {
	my ($id, $score, $reason) = @_;
	my ($best, $current);

	if ($score == 0) {
		if ($spree_current[$id] > $spree_best[$id]) {
			$spree_best[$id] = $spree_current[$id];
			$spree_reason[$id] = $reason;
		}
		$spree_current[$id] = 0;
	} else {
		$spree_current[$id] += $score;
	}
}



sub parse_action {
}



# create_game:
# 0	gametype
# 1	map
# 2	length
# 3	players
# ret	game id, gametype id
#
sub create_game {
	my ($gametype, $map, $length, $players) = @_;
	my $time;
	my ($gametype_id, $map_id);
	my ($query, $result, @row);
	my $game_id;

	$gametype_id = get_gametype_id($gametype);
	$map_id = get_map_id($map);

	$time = localtime();
	DB::transaction_start();
	$game_id = DB::get_sequence("game_id_seq");
	$query = "INSERT INTO game" .
			" (id, gametype, map, length," .
			" timestamp, players)" .
			" VALUES" .
			" ($game_id, $gametype_id, $map_id, $length," .
			" '$time', $players)";
	DB::update($query);
	DB::transaction_end();

	return ($game_id, $gametype_id);
}



# get_gametype_id:
# 0	gametype
# ret	gametype id
#
sub get_gametype_id {
	my $gametype = $_[0];
	my $id;
	my ($query, $result, @row);
	
	$query = "SELECT id FROM gametype WHERE type = '$gametype'";
	$result = DB::query($query);
	@row = $result->fetchrow;
	if ($#row == -1) {
		# Not in database: create new player
		DB::transaction_start();
		$id = DB::get_sequence("gametype_id_seq");
		$query = "INSERT INTO gametype (id, type)" .
				" VALUES ($id, '$gametype')";
		DB::update($query);
		DB::transaction_end();
	} else {
		$id = $row[0];
	}

	return $id;
}



# get_map_id:
# 0	map
# ret	map id
#
sub get_map_id {
	my $map = $_[0];
	my $id;
	my ($query, $result, @row);
	
	$query = "SELECT id FROM map WHERE map = '$map'";
	$result = DB::query($query);
	@row = $result->fetchrow;
	if ($#row == -1) {
		# Not in database: create new player
		DB::transaction_start();
		$id = DB::get_sequence("map_id_seq");
		$query = "INSERT INTO map (id, map) VALUES ($id, '$map')";
		DB::update($query);
		DB::transaction_end();
	} else {
		$id = $row[0];
	}

	return $id;
}



# get_player_rating:
# 0	player
# 1	gametype id
# ret	rating
#
sub get_player_rating {
	my ($player, $gametype) = @_;
	my $rating;
	my ($query, $result, @row);

	return $base_rating if ($player == -1);
	
	$query = "SELECT rating FROM player_stat WHERE player = '$player'" .
			" AND gametype = $gametype";
	$result = DB::query($query);
	@row = $result->fetchrow;
	if ($#row == -1) {
		# Not in database: create stats for player
		$rating = $base_rating;
		$query = "INSERT INTO player_stat (player, gametype, rating)" .
				" VALUES ($player, $gametype, $rating)";
		DB::update($query);
	} else {
		$rating = $row[0];
	}

	return $rating;
}
