package DB;

use Pg;
require "sys/syscall.ph";
do "common.pl";

my $TIMEVAL_T = "LL";
my ($done, $start);
$done = $start = pack($TIMEVAL_T, ());
syscall(&SYS_gettimeofday, $start, 0);



$SIG{PIPE} = sub { print "Broken pipe!\n"; exit(0); };

my $option_ref = Pg::conndefaults();
my $conn;

my $hostaddr = "dbserver";
my $dbname = "dbname";
my $user = "dbuser";
my $password = "dbpass";



# sql_connect:
#
sub connect {
	my $cmd = "hostaddr=$hostaddr dbname='$dbname'" .
			" user=$user password=$password";
	$conn = Pg::connectdb($cmd);
	die $conn->errorMessage if ($conn->status ne PGRES_CONNECTION_OK);
}



# sql_update:
# 0	command
#
sub update {
	my $query = $_[0];
	my $result;

	DB::connect() if (! defined($conn));
	$result = $conn->exec($query);
	if ($result->resultStatus ne PGRES_COMMAND_OK) {
		print STDERR "QUERY FAILED: $query\n";
		die $conn->errorMessage;
	}
}



# sql_query
# 0	command
# ret	$result
#
sub query {
	my $query = $_[0];
	my $result;

	DB::connect() if (! defined($conn));
	$result = $conn->exec($_[0]);
	if ($result->resultStatus ne PGRES_TUPLES_OK) {
		print STDERR "QUERY FAILED: $query\n";
		die $conn->errorMessage;
	}

	return $result;
}



# sql_get_sequence
# 0	sequence name
# ret	$result
#
sub get_sequence {
	my $sequence = $_[0];
	my ($cmd, $result, @row);

	DB::connect() if (! defined($conn));
	$cmd = "SELECT nextval('$sequence') FROM $sequence";
	$result = query($cmd);
	@row = $result->fetchrow;
	return $row[0];
}



# sql_transact_start
#
sub transaction_start {
	update("BEGIN");
}



# sql_transaction_end
#
sub transaction_end {
	update("END");
}



# safe_string
# 0	string
# ret	safe string
#
sub safe_string {
	$_ = $_[0];
	s/'//g;
	return $_;
}



# write_access_log
# ret	time taken
#
sub write_access_log {
	syscall(&SYS_gettimeofday, $done, 0);
	my @tstart = unpack($TIMEVAL_T, $start);
	my @tdone  = unpack($TIMEVAL_T, $done);
	
	for ($tdone[1], $tstart[1]) {
		$_ /= 1000000;
	}
	my $ndelta = ($tdone[0] + $tdone[1]) - ($tstart[0] + $tstart[1]);

	my $query = sprintf("INSERT INTO access_log" .
				" (client, ip, script, query, time)" .
				" VALUES ('%s', '%s', '%s', '%s', %f)",
			safe_string(Common::safe_string(
					$ENV{"HTTP_USER_AGENT"})),
			safe_string(Common::safe_string($ENV{"REMOTE_ADDR"})),
			safe_string(Common::safe_string($ENV{"SCRIPT_NAME"})),
			safe_string(Common::safe_string($ENV{"QUERY_STRING"})),
			$ndelta);
	update($query);
	return $ndelta;
}
