package Common;

use strict;
use warnings FATAL => 'all';



# safe_number:
# 0	number
# 1	default
# ret	number
#
sub safe_number {
	my $ret = $_[0];;
	$ret = $_[1] if (! defined($ret));
	return 0 if (! defined($ret));
	return 0 if ($ret =~ /[^0-9\.\-]/);
	return $ret;
}



# safe_string:
# 0	string
# 1	default
#
sub safe_string {
	my $ret = $_[0];
	$ret = $_[1] if (! defined($ret));
	$ret = "" if (! defined($ret));
	return $ret;
}



# count_efficiency
# 0	kills
# 1	deaths
# 2	suicides
# 3	teamkills
#
sub count_efficiency {
	my $k = safe_number($_[0]);
	my $d = safe_number($_[1]);
	my $s = safe_number($_[2]);
	my $tk = safe_number($_[3]);
	my $div = $k + $d + $s + $tk;

	return 0 if ($div == 0);
	return $k / $div * 100;
}



# round
# 0	value
# 1	decimals
# 2	force prefix [-+]
#
sub round {
	my $val = safe_number($_[0]);
	my $dec = safe_number($_[1]);
	my $force = defined($_[2]) ? 1 : 0;
	$val += 5 / (10 ** ($dec + 1));
	my $prefix = $force == 1 && $val > 0 ? "+" : "";

	return int($val) if ($dec == 0);

	if ($val == int($val)) {
		$val .= ".0000000000000000";
	} else {
		$val .= "0000000000000000";
	}
	$val =~ s/(\d+\.\d{$dec}).*/$1/;
	return "$prefix$val";
}



# seconds_to_time
# 0	seconds
#
sub seconds_to_time {
	my $t;
if (0 == 1) {
	my $time = $_[0];
	$t = $time / 60 + 0.5;
	$t = "00" . (int($t % 60));
	$t =~ s/.*(..)/$1/;
	$t = "" . (int($time / 3600)) . ":$t";
} else {
	my ($h, $m, $s);
	my $time = $_[0];

	$s = $time % 60;
	$m = int($time / 60) % 60;
	$h = int($time / 3600);

	$t = "";
	$t = "$h" if ($h > 0);
	$t .= " $m'" if ($m > 0 || $h > 0);
	$t .= " $s''" if ($s > 0 || $m > 0 || $h > 0);
	$t = "0''" if ($t eq "");
}

	return $t;
}



# safe_string
# 0	string
# ret	html-safe string
#
sub html_safe_string {
	$_ = $_[0];
	s/&/&amp;/g;
	s/</&lt;/g;
	s/>/&gt;/g;
	return $_;
}



# verticalize_string
# 0	string
# 1	non-html
# ret	string verticalized
#
sub verticalize_string {
	my $str = $_[0];
	my $html = defined($_[1]) ? 0 : 1;
	$str =~ s/(.)/$1\n/g;
	$str = safe_string($str) if ($html == 1);
	$str =~ s/\n/<br \/>/g;
	return $str;
}
