#! /usr/bin/perl

use strict;
use warnings FATAL => 'all';


my $self = "game.cgi";
my $show_games = 9;

do "db.pl";
do "common.pl";
do "html.pl";
do "stats.pl";

my %nick_cache;



# parameters taken:
#	p	player	player id

# This is insane: we need to use HTML::get twice to keep Perl happy
my $game = Common::safe_number($HTML::get{"g"});
$game = Common::safe_number($HTML::get{"g"});

HTML::bad_parameters() if ($game == 0);



my ($query, $result, @row);

my ($gametype, $map, $length, $timestamp, $nplayers) =
	Stats::get_game_info($game);



# Player summary
$query = "SELECT player, kills, deaths, suicides," .
		" rating, rating_change, time" .
		" FROM game_player WHERE game = $game";
$result = DB::query($query);
HTML::bad_parameters() if ($result->ntuples == 0);


HTML::write_head("Match summary for game #$game");

printf("<h1>A %s game #%s in %s with %d players</h1>\n",
		$gametype, $game, $map, $nplayers);
printf("<p class=\"loginfo\">Logged in DB: $timestamp</p>\n");

print <<EOF
<h2>Player summary</h2>
<table>
<tr>
	<th>Rank</th>
	<th>Player</th>
	<th>K</th>
	<th>D</th>
	<th>S</th>
	<th>Eff. %</th>
	<th>Avg KPH</th>
	<th>Avg TTL</th>
	<th>Rating</th>
	<th>Rating change</th>
	<th>Time</th>
</tr>
EOF
;

my @unsorted_players;
my (%skills, %sdeaths, %ssuicides, %srating, %sratingch, %stime);
while (my ($player, $kills, $deaths, $suicides,
		$rating, $ratingch, $time) = $result->fetchrow) {
	push(@unsorted_players, $player);
	$skills{$player} = $kills;
	$sdeaths{$player} = $deaths;
	$ssuicides{$player} = $suicides;
	$srating{$player} = $rating;
	$sratingch{$player} = $ratingch;
	$stime{$player} = $time;
}

my @players;
foreach $_ (@unsorted_players) {
	push(@players, $_);
}
for (my $i = 0; $i <= $#players - 1; $i++) {
	my $bk = Common::safe_number($skills{$players[$i]});
	my $bd = Common::safe_number($sdeaths{$players[$i]});
	my $bs = Common::safe_number($ssuicides{$players[$i]});
	my $bi = $i;
	for (my $j = $i + 1; $j <= $#players; $j++) {
		my $better = 0;
		my $tk = Common::safe_number($skills{$players[$j]});
		my $td = Common::safe_number($sdeaths{$players[$j]});
		my $ts = Common::safe_number($ssuicides{$players[$j]});
		if ($tk > $bk) {
			$better = 1;
		} elsif ($tk == $bk) {
			if ($td < $bd) {
				$better = 1;
			} elsif ($td == $bd) {
				if ($ts < $bs) {
					$better = 1;
				}
			}
		}
		if ($better == 1) {
			$bk = $tk;
			$bd = $td;
			$bs = $ts;
			$bi = $j;
		}
	}
	if ($bi != $i) {
		my $t = $players[$i];
		$players[$i] = $players[$bi];
		$players[$bi] = $t;
	}
}

my $true_rank = 1;
my $last_kills = -1;
my $last_rank = 1;
foreach my $killer (@players) {
	my $nick = Common::html_safe_string(get_nick($killer));
	my $kills = $skills{$killer};
	my $deaths = $sdeaths{$killer};
	my $suicides = $ssuicides{$killer};
	my $eff = Common::round(Common::count_efficiency(
			$kills, $deaths, $suicides), 2);
	my $rtime = $stime{$killer};
	my $kph = Common::round(
			$kills / (($rtime == 0 ? 1 : $rtime) / 3600), 2);
	my $ttl = Common::round($rtime / ($deaths + 1), 1);
	my $rating = Common::round($srating{$killer}, 2);
	my $ratingch = Common::round($sratingch{$killer}, 2, 1);
	my $time = Common::seconds_to_time($rtime);
	my $rank;
	if ($kills == $last_kills) {
		$rank = $last_rank;
	} else {
		$rank = $true_rank;
		$last_kills = $kills;
	}
	$last_rank = $rank;
	$true_rank++;
	print <<EOF
<tr>
	<td>$rank</td>
	<td><a href="player.cgi?p=$killer">$nick</a></td>
	<td>$kills</td>
	<td>$deaths</td>
	<td>$suicides</td>
	<td>$eff</td>
	<td>$kph</td>
	<td>$ttl</td>
	<td>$rating</td>
	<td>$ratingch</td>
	<td>$time</td>
</tr>
EOF
;
}
print "</table>\n";



# Sort players by nick
@players = ();
foreach $_ (@unsorted_players) {
	push(@players, $_);
}
for (my $i = 0; $i <= $#players - 1; $i++) {
	my $nick = get_nick($players[$i]);
	my $ti = $i;
	for (my $j = $i + 1; $j <= $#players; $j++) {
		my $tnick = get_nick($players[$j]);
		if ($tnick lt $nick) {
			$nick = $tnick;
			$ti = $j;
		}
	}
	if ($ti != $i) {
		my $t = $players[$i];
		$players[$i] = $players[$ti];
		$players[$ti] = $t;
	}
}

$query = "SELECT killer, victim, kills FROM game_kills WHERE game = $game";
$result = DB::query($query);

my %players;	# {id} => 1
my %killcache;	# {"id \n id"} => kills

while (my ($killer, $victim, $kills) = $result->fetchrow) {
	$killcache{"$killer\n$victim"} = $kills;
}

# Print column headers
print <<EOF
<h2>Kills Match Up (Efficiency%)</h2>
<table>
<tr>
	<th>Victim<br />\\<br />Killer</th>
EOF
;

foreach my $victim (@players) {
	my $nick = Common::verticalize_string(get_nick($victim));
	printf("\t<th><a href=\"player.cgi?p=%d\">%s</a></th>\n",
			$victim, $nick);
}

printf("\t<th class=\"total\">%s</th>\n", Common::verticalize_string("Total"));
print "</tr>\n";

# Print rows
foreach my $killer (@players) {
	my $nick = Common::html_safe_string(get_nick($killer));
	print "<tr>\n";
	printf("<th><a href=\"player.cgi?p=%d\">%s</a></th>\n", $killer, $nick);
	foreach my $victim (@players) {
		my $class = $killer == $victim ? " class=\"suicide\"" : "";
		my $eff = Common::round(Common::count_efficiency(
				$killcache{"$killer\n$victim"},
				$killcache{"$victim\n$killer"}), 2);
		printf("\t<td%s>%d (%s)</td>\n", $class,
				Common::safe_number(
					$killcache{"$killer\n$victim"}),
				$eff);
	}
	printf("\t<td class=\"total\">%d</td>\n",
			Common::safe_number($skills{$killer}));
	printf("</tr>\n");
}

print "<tr>\n";
print "\t<th class=\"total\">Total</th>\n";
# Print other totals
foreach my $victim (@players) {
	printf("<td class=\"total\">%d</td>\n",
			Common::safe_number($sdeaths{$victim}));
}
print <<EOF
	<td>&nbsp;</td>
</tr>
</table>
EOF
;


$query = "SELECT last_game FROM quick_info";
$result = DB::query($query);
my ($last_game) = $result->fetchrow;

print <<EOF
<h2>Other Games</h2>

<ul>
EOF
;
my $first = $game - int($show_games / 2);
my $last = $game + int($show_games / 2);
if ($first < 0) {
	$last -= $first;
	$first = 0;
} elsif ($last >= $last_game) {
	$first -= $last - $last_game;
	$last = $last_game;
}
$first = 0 if ($first < 0);

print "<li>...</li>" if ($first != 0);
for (my $i = $first; $i <= $last; $i++) {
	print "<li>";
	my ($gt, $map, $len, $time, $pla) = Stats::get_game_info($i);
	my $info = "#$i: $map / $gt, $pla players ($time)";
	if ($i != $game) {
		printf("<a href=\"%s?g=%d\">%s</a> ", $self, $i, $info);
	} else {
		print "$info";
	}
	print "</li>\n";
}
print "<li>...</li>\n" if ($last != $last_game);
print "</ul>\n";



HTML::write_tail();



# get_nick:
# 0	player id
# ret	nick
#
sub get_nick {
	my ($id) = @_;
	my $nick = $nick_cache{$id};
	return $nick if (defined($nick));

	my ($query, $result, @row);
	$query = "SELECT name FROM player WHERE id = $id";
	$result = DB::query($query);
	HTML::broken_database("'game' says we have player $id, 'player'" .
			" disagrees") if ($result->ntuples == 0);
	($nick) = $result->fetchrow;
	$nick_cache{$id} = $nick;
	return $nick;
}
