#! /usr/bin/perl

use strict;
use warnings FATAL => 'all';

do "db.pl";
do "html.pl";
do "common.pl";

use GD;

my $font = gdSmallFont;

my $width = 320;
my $height = 192;
my $pad_side = 2;
my $pad_left = $pad_side;
my $pad_right = $pad_side + $font->width * 3;
my $pad_top = 2 + $font->height;
my $pad_bottom = 2;

my $baseline = 100;


my $player = Common::safe_number($HTML::get{"p"});
my $gametype = Common::safe_number($HTML::get{"g"});
my $global_fit = Common::safe_number($HTML::get{"f"}, 1);
my $draw_baseline = Common::safe_number($HTML::get{"b"}, 0);
my $size = Common::safe_number($HTML::get{"z"}, 0);
my $grid = Common::safe_number($HTML::get{"x"}, 1);
my $seek = Common::safe_number($HTML::get{"s"}, 0);
my $omit = Common::safe_number($HTML::get{"o"}, 0);

HTML::bad_parameters() if ($size < 0 || $size > 1);
HTML::bad_parameters() if ($player == 0 || $gametype == 0);
HTML::bad_parameters() if ($draw_baseline == 1 && $grid == 1);

if ($size == 0) {
	$width = 320;
	$height = 200;
} elsif ($size == 1) {
	$width = 800;
	$height = 600;
}

my $image = new GD::Image($width, $height);

my $col_white = $image->colorAllocate(255, 255, 255);
my $col_black = $image->colorAllocate(0, 0, 0);
my $col_red = $image->colorAllocate(255, 0, 0);
my $col_blue = $image->colorAllocate(0, 0, 255);

# $image->transparent($col_white);
$image->interlaced("true");

$image->rectangle(0, 0, $width - 1, $height - 1, $col_blue);

binmode STDOUT;



my ($query, @row);
my $player_rating;

# Get player rating
$query = "SELECT gp.rating FROM game_player AS gp, game, gametype AS gt" .
		" WHERE gp.player = $player AND game.gametype = $gametype" .
		" AND gp.game = game.id AND game.gametype = gt.id";
$player_rating = DB::query($query);
my $games = $player_rating->ntuples + 1;

HTML::bad_parameters() if ($games == 1);


my @rating;
my ($low, $high);

$rating[0] = $baseline;

# Get global hi/low in rating
if ($global_fit == 1) {
	$query = "SELECT rating_low, rating_high FROM gametype" .
			" WHERE id = $gametype";
	my $result = DB::query($query);
	($low, $high) = $result->fetchrow;
} else {
	$low = $high = $rating[0];
}

my $i = 1;
while (($rating[$i]) = $player_rating->fetchrow) {
	if ($global_fit == 0) {
		if ($rating[$i] > $high) {
			$high = $rating[$i];
		} elsif ($rating[$i] < $low) {
			$low = $rating[$i];
		}
	}
	$i++;
}

# Get stepped limits for rating high/low
$low = int($low / 20) * 20;
$high = int($high / 20 + 1) * 20;

my $x_mul = ($width - $pad_left - $pad_right - 2) / ($games - 1);
my $y_mul = ($height - $pad_top - $pad_bottom) / ($high - $low);


printf("Content-type: image/png\n\n");

# Baseline
if ($draw_baseline == 1 && $low < 100 && $high > 100) {
	draw_grid_line(100, $col_blue);
} else {
	my $interval = $font->height * 2 / (($height - $pad_bottom - $pad_top) /
			($high - $low));
	$interval = int($interval / 10 + 1) * 10;
	my $start = int($low / $interval + 1) * $interval;
	my $adjust = $baseline % $interval;
	if (get_scaled_y($start - $adjust) < $height - $pad_bottom) {
		$start -= $adjust;
	} else {
		$start += $interval - $adjust;
	}
	for (my $i = $start; $i < $high; $i += $interval) {
		draw_grid_line($i, $col_black);
	}
}

my ($last_x, $last_y);
my ($this_x, $this_y);
($last_x, $last_y) = scale_coords(0, $rating[0]);

for ($i = 1; $i < $games; $i++) {
	($this_x, $this_y) = scale_coords($i, $rating[$i]);
	$image->line($last_x, $last_y, $this_x, $this_y, $col_red);
	$last_x = $this_x;
	$last_y = $this_y;
}

# $image->string(gdSmallFont, $pad_left + 4, $pad_top + 4, "Progress", $col_black);
#$image->string($font, $width - $pad_right, 4, "100", $col_black);


# Add caption
$query = "SELECT type FROM gametype WHERE id = $gametype";
my $gametype_res = DB::query($query);
my ($gametype_str) = $gametype_res->fetchrow;
$image->string($font, $pad_left + 2, 2, "$gametype_str", $col_black);

print $image->png;
DB::write_access_log();



# scale_coords
# 0	x
# 1	y
#
sub scale_coords {
	my $x = $_[0];
	my $y = $_[1];
	
	my $rx = $x * $x_mul + $pad_left;
	my $ry = get_scaled_y($y);

	return ($rx, $ry);
}



# get_scaled_y
# 0	y
#
sub get_scaled_y {
	return $height - $pad_bottom - ($_[0] - $low) * $y_mul;
}




# draw_grid_line
# 0	rating
# 1	color
#
sub draw_grid_line {
	my $y = $_[0];
	my $color = $_[1];
	my ($dx, $sy);
	
	($dx, $sy) = scale_coords(0, $y);
	$image->line($pad_side, $sy, $width - $pad_side, $sy, $color);
	$image->string($font, $width - $pad_right, $sy - $font->height,
			int($y), $color);
}
