#! /usr/bin/perl

use strict;
use warnings FATAL => 'all';


my $self = "player.cgi";
my $maxgames = 16;

do "db.pl";
do "common.pl";
do "html.pl";
do "stats.pl";

my ($query, $result, @row);

# parameters taken:
#	p	player	player id

my $player = Common::safe_number($HTML::get{"p"}, 0);
my $global_fit = Common::safe_number($HTML::get{"f"}, 0);
my $graph_gametype = Common::safe_number($HTML::get{"g"}, 0);

my $nick;

HTML::bad_parameters() if ($player == 0);



$query = "SELECT name FROM player_alias WHERE id = $player ORDER BY name";
$result = DB::query($query);
my $alias_count = $result->ntuples;

HTML::bad_parameters() if ($alias_count == 0);



HTML::write_head("Player statistics for player #$player");


# Aliases
printf("<h1>Career summary for player #%d</h1>\n", $player);

if ($alias_count > 1) {
	print "<p>Used nicks:\n";
	print "<ul>\n";

	while (my ($alias) = $result->fetchrow) {
		printf("<li>%s</li>", $alias);
	}
	printf "</ul></p>\n";
}


# Cache gametypes
my @gametype_cache;
$query = "SELECT id, type FROM gametype";
$result = DB::query($query);
while (my ($id, $type) = $result->fetchrow) {
	$gametype_cache[$id] = $type;
}



# Gametype summary
print <<EOF
<h2>Per gametype</h2>

<table>
<tr>
	<th>Game Type</th>
	<th>Rating</th>
	<th>Ranking</th>
	<th>K</th>
	<th>D</th>
	<th>S</th>
	<th>Eff. %</th>
	<th>Avg. KPH</th>
	<th>Avg. TTL</th>
	<th>Matches</th>
	<th>Time</th>
	<th>Graph</th>
</tr>
EOF
;

$query = "SELECT gametype, kills, deaths, suicides," .
		" time, games, rating" .
		" FROM player_stat WHERE player = $player" .
		" ORDER BY gametype";
$result = DB::query($query);


while (my ($gametype_id, $kills, $deaths, $suicides, $time, $matches, $rating) =
		$result->fetchrow) {
	my ($ranking, $players, $eff, $kph, $ttl);
	$graph_gametype = $gametype_id if ($graph_gametype == 0);

	($players, $ranking) = Stats::get_ranking($gametype_id, $player);
	$rating = Common::round($rating, 2);
	$eff = Common::round(Common::count_efficiency(
			$kills, $deaths, $suicides), 2);
	$kph = Common::round($kills / (($time == 0 ? 1 : $time) / 3600), 2);
	$ttl = Common::seconds_to_time($time / ($deaths + 1));
	$time = Common::seconds_to_time($time);
	my $gametype = $gametype_cache[$gametype_id];
	print <<EOF
<tr>
	<td>$gametype</td>
	<td>$rating</td>
	<td>$ranking ($players)</td>
	<td>$kills</td>
	<td>$deaths</td>
	<td>$suicides</td>
	<td>$eff</td>
	<td>$kph</td>
	<td>$ttl</td>
	<td>$matches</td>
	<td>$time</td>
	<td><a href="$self?p=$player&amp;g=$gametype_id&amp;f=0">p</a> /
	<a href="$self?p=$player&amp;g=$gametype_id&amp;f=1">g</a></td>
</tr>
EOF
;
}

printf("</table>\n");

my $gametype_str = $gametype_cache[$graph_gametype];
print <<EOF
<div class="graph">
<a href="img_rating.cgi?p=$player&amp;g=$graph_gametype&amp;f=$global_fit&amp;z=1">
<img src="img_rating.cgi?p=$player&amp;g=$graph_gametype&amp;f=$global_fit&amp;z=0"
		width="320" height="200" alt="$gametype_str graph" /></a>
</div>
EOF
;

# Played games
print "<h2>Latest games played</h2>\n";

$query = "SELECT game FROM game_player WHERE player = $player" .
		" ORDER BY game DESC";
$result = DB::query($query);
print "<ul>\n";
for (my $i = 0; $i < $maxgames && $i < $result->ntuples; $i++) {
	my ($game) = $result->fetchrow;
	print "<li>";
	my ($gt, $map, $len, $time, $pla) = Stats::get_game_info($game);
	my $info = "#$game: $map / $gt, $pla players ($time)";
	printf("<a href=\"game.cgi?g=%d\">" .
			"#%d: %s / %s, %d players (%s)</a> ",
			$game,
			$game, $map, $gt, $pla, $time);
	print "</li>\n";
}
print "</ul>\n";


HTML::write_tail();
