#! /usr/bin/perl

use strict;
use warnings FATAL => 'all';


my $self = "server.cgi";

do "db.pl";
do "common.pl";
do "html.pl";
do "stats.pl";

HTML::write_head("Server performance statistics");

my ($query, $result);

$query = "SELECT client, ip, script, time FROM access_log";
$result = DB::query($query);

my (%client, %ip, %script, %script_time);
my ($time, $rows);

$time = 0;
$rows = $result->ntuples;

while (my ($tclient, $tip, $tscript, $ttime) = $result->fetchrow) {
	$time += $ttime;
	$client{$tip} = $tclient;
	$ip{$tip}++;
	$script{$tscript}++;
	$script_time{$tscript} += $ttime;
}

{ # block
my $ftime = sprintf("%.3f", $time);
my $atime = sprintf("%.3f", $time / ($rows > 0 ? $rows : 1));
print <<EOF
<h1>Server access statistics</h1>

<p>Server accessed $rows time(s), total time took $ftime seconds, average
time taken in page generation $atime seconds.</p>
EOF
;
} # block

{ # block
print <<EOF
<h2>Time by script</h2>

<table>
<tr>
	<th>Script</th>
	<th>Accessed (times)</th>
	<th>Time</th>
	<th>Average time (s)</th>
</tr>
EOF
;

foreach my $sscript (keys %script) {
	my $ttimes = $script{$sscript};
	my $ttime = Common::seconds_to_time(int($script_time{$sscript}));
	my $tatime = sprintf("%.3f", $script_time{$sscript} / $ttimes);
	print <<EOF
<tr>
	<td>$sscript</td>
	<td>$ttimes</td>
	<td>$ttime</td>
	<td>$tatime</td>
</tr>
EOF
;
}
print <<EOF
</table>
EOF
;
} # block

{ # block
my %client_sum;
while (my ($cip, $cclient) = each %client) {
	$client_sum{$cclient}++;
}
print <<EOF
<h2>Access by browser</h2>

<table>
<tr>
	<th>Browser</th>
	<th>IPs</th>
</tr>
EOF
;
while (my ($client, $times) = each %client_sum) {
	print <<EOF
<tr>
	<td>$client</td>
	<td>$times</td>
</tr>
EOF
;
}
print "</table>\n";
} # block



HTML::write_tail();
